DELIMITER $$
DROP PROCEDURE IF EXISTS create_reservation;

CREATE DEFINER =`kea`@`%` PROCEDURE `create_reservation`(
  IN  hwaddress        varchar(18),
  IN  dhcp4_subnet_id  int(10) unsigned,
  IN  ipv4_reservation varchar(16),
  IN  hostname         varchar(255),
  IN  server_hostname  varchar(64),
  IN  boot_file_name   varchar(128),
  OUT host_id          int(10) unsigned
)

  proc: BEGIN

    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
    BEGIN
      ROLLBACK;
    END;

    START TRANSACTION;
    SET @identifier_type = 'hw-address';
    SET @dhcp4_subnet_id = 1;
    SET @boot_file_name = 'bootfile.efi';

    SET @dhcp_identifier := UNHEX(REPLACE(hwaddress, ':', ''));

    IF EXISTS(SELECT host_id
              FROM hosts
              WHERE dhcp_identifier = @dhcp_identifier)
    THEN
      UPDATE hosts
      SET hostname            = hostname,
        ipv4_address          = INET_ATON(ipv4_reservation),
        dhcp4_server_hostname = server_hostname,
        dhcp4_boot_file_name  = boot_file_name
      WHERE
        dhcp_identifier = @dhcp_identifier;

      COMMIT;
      LEAVE proc;
    END IF;

    INSERT INTO hosts (dhcp_identifier,
                       dhcp_identifier_type,
                       dhcp4_subnet_id,
                       ipv4_address,
                       hostname,
                       dhcp4_next_server,
                       dhcp4_server_hostname,
                       dhcp4_boot_file_name)
    VALUES (
      UNHEX(REPLACE(hwaddress, ':', '')),
      (SELECT type
       FROM host_identifier_type
       WHERE name = @identifier_type),
      dhcp4_subnet_id,
      INET_ATON(ipv4_reservation),
      hostname,
      NULL,
      server_hostname,
      boot_file_name
    );
    COMMIT;

  END;
